@@ -39,14 +39,6 @@
 #include <openssl/x509.h>
 #include <openssl/ssl.h>
 
-#ifdef HAVE_SOCKS_H
-#include <socks.h>
-#endif 
-
-#ifdef NEED_ADDRINFO_H
-#include "addrinfo.h"
-#endif
-
 #include <sys/time.h>
 #include <sys/socket.h>
 #include <sys/file.h>
@@ -57,8 +49,11 @@
 #include <signal.h>
 #include <fcntl.h>
 #include <netinet/in.h>
+#ifdef HAVE_POLL_H
+#include <sys/poll.h>
+#endif
 #define _GNU_SOURCE
-#include <getopt.h>
+#include "getopt.h"
 
 static SSL_CTX *tls_ctx = NULL;
 static SSL *tls_conn = NULL;
@@ -136,13 +131,22 @@ tls_connect (hostname, service)
      const char *hostname, *service;
 {
   struct protoent *proto;
+  int server, false = 0, family, socktype;
+  struct sockaddr *addr;
+  struct sockaddr_in sin;
+  socklen_t addrlen;
+#ifdef HAVE_ADDRINFO
   struct addrinfo *in, hints;
-  int server, false = 0;
+#else
+  struct hostent *host;
+  struct servent serv;
+#endif
 
   proto = getprotobyname ("tcp");
   if (!proto)
     return -1;
 
+#ifdef HAVE_ADDRINFO
   memset (&hints, 0, sizeof (hints));
   hints.ai_family = AF_UNSPEC;
   hints.ai_socktype = SOCK_STREAM;
@@ -151,15 +155,36 @@ tls_connect (hostname, service)
   if (getaddrinfo (hostname, service, &hints, &in) < 0) 
     return -1;
 
-  server = socket (in->ai_family, in->ai_socktype, 0);
-  if (server < 0)
+  family = in->ai_family;
+  socktype = in->ai_socktype;
+  addr = in->ai_addr;
+  addrlen = in->ai_addrlen;
+  freeaddrinfo (in);
+#else
+  memset (&sin, 0, sizeof (sin));
+  host = gethostbyname (hostname);
+  if (!host)
+    return -1;
+  memcpy (&sin.sin_addr, host->h_addr, host->h_length);
+  serv = getservbyname (service, proto->p_name);
+  if (serv)
+    sin.sin_port = htons (serv->s_port);
+  else if (isdigit (service[0]))
+    sin.sin_port = htons (atoi (service));
+  family = sin.sin_family = AF_INET;
+  socktype = SOCK_STREAM;
+  addr = (struct sockaddr *)&sin;
+  addrlen = sizeof (sin);
+#endif
+
+  server = socket (family, socktype, 0);
+  if (server == -1)
     return -1;
 
-  if (setsockopt (server, SOL_SOCKET, SO_KEEPALIVE,
-		  (const char *) &false, sizeof (false))) 
-    return -1;
+  setsockopt (server, SOL_SOCKET, SO_KEEPALIVE, (const char *) &false,
+	      sizeof (false));
 
-  if (connect (server, in->ai_addr, in->ai_addrlen) < 0)
+  if (connect (server, addr, addrlen) < 0)
     {
       close (server);
       return -1;
@@ -202,7 +227,11 @@ main (argc, argv)
 {
   int in = fileno (stdin), out = fileno (stdout), 
     nbuffer, wrote;
+#ifdef HAVE_POLL
+  struct pollfd readfds[2], writefds[1];
+#else
   fd_set readfds, writefds;
+#endif
   char buffer[BUFSIZ], *retry;
   struct sigaction act;
 
@@ -218,7 +247,7 @@ main (argc, argv)
 
   while (1)
     {
-      c = getopt_long (argc, argv, "c:k:v:f", long_options, &option_index);
+      c = getopt_long (argc, argv, "c:k:v:", long_options, &option_index);
       if (c == -1)
 	break;
     
@@ -258,17 +287,34 @@ main (argc, argv)
   act.sa_flags = SA_RESTART|SA_RESETHAND;
   sigaction (SIGALRM, &act, NULL);
 
+#ifdef HAVE_POLL
+  readfds[0].fd = in;
+  readfds[1].fd = tls_fd;
+  readfds[0].events = POLLIN;
+  readfds[1].events = POLLIN;
+  writefds[0].events = POLLOUT;
+#endif
+
   while (1)
     {
+#ifdef HAVE_POLL
+      if (poll (readfds, 2, -1) == -1 && errno != EINTR)
+#else
+      FD_ZERO (&readfds);
       FD_SET (tls_fd, &readfds);
       FD_SET (in, &readfds);
       if (select (tls_fd+1, &readfds, NULL, NULL, NULL) == -1
 	  && errno != EINTR )
+#endif
 	{
-	  perror ("select");
+	  perror ("poll");
 	  return 1;
 	}
+#ifdef HAVE_POLL
+      if (readfds[0].revents & POLLIN)
+#else
       if (FD_ISSET (in, &readfds))
+#endif
 	{
 	  nbuffer = read (in, buffer, sizeof buffer -1);
 
@@ -276,10 +322,16 @@ main (argc, argv)
 	    goto finish;
 	  for (retry = buffer; nbuffer > 0; nbuffer -= wrote, retry += wrote)
 	    {
+#ifdef HAVE_POLL
+	      writefds[0].fd = tls_fd;
+	      if (poll (writefds, 1, -1) == -1)
+#else
+	      FD_ZERO (&writefds);
 	      FD_SET (tls_fd, &writefds);
 	      if (select (tls_fd+1, NULL, &writefds, NULL, NULL) == -1)
+#endif
 		{
-		  perror ("select");
+		  perror ("poll");
 		  return 1;
 		}
 	      if (tls_conn) 
@@ -289,7 +341,11 @@ main (argc, argv)
 	      if (wrote < 0) goto finish;
 	    }
 	}
+#ifdef HAVE_POLL
+      if (readfds[1].revents & POLLIN)
+#else
       if (FD_ISSET (tls_fd, &readfds))
+#endif
 	{
 	  if (tls_conn)
 	    nbuffer = SSL_read (tls_conn, buffer, sizeof buffer -1);
@@ -299,10 +355,16 @@ main (argc, argv)
 	    goto finish;
 	  for (retry = buffer; nbuffer > 0; nbuffer -= wrote, retry += wrote)
 	    {
+#ifdef HAVE_POLL
+	      writefds[0].fd = out;
+	      if (poll (writefds, 1, -1) == -1)
+#else
+	      FD_ZERO (&writefds);
 	      FD_SET (out, &writefds);
 	      if (select (out+1, NULL, &writefds, NULL, NULL) == -1)
+#endif
 		{
-		  perror ("select");
+		  perror ("poll");
 		  return 1;
 		}
 	      wrote = write (out, retry, nbuffer);
